@@ -5,6 +5,7 @@ from airflow.utils.decorators import apply_defaults
 from ewah.dwhooks import get_dwhook
 from ewah.constants import EWAHConstants as EC
 
+import hashlib
 
 class EWAHBaseOperator(BaseOperator):
     """Extension of airflow's native Base Operator.
@@ -57,6 +58,11 @@ class EWAHBaseOperator(BaseOperator):
 
     _REQUIRES_COLUMNS_DEFINITION = False # raise error if true an none supplied
 
+    _INDEX_QUERY = '''
+        CREATE INDEX IF NOT EXISTS {0}
+        ON "{1}"."{2}" ({3})
+    '''
+
     upload_call_count = 0
 
     _metadata = {} # to be updated by operator, if applicable
@@ -78,9 +84,17 @@ class EWAHBaseOperator(BaseOperator):
         add_metadata=True, # adds columns with metadata to all rows
         # Note: that metadata is specified by a dict on operator level!
         # metadata can only be added if no columns definition is given
+        exclude_columns=[], # list of columns to exclude, if no
+        # columns_definition was supplied (e.g. for select * with sql)
+        index_columns=[], # list of columns to create an index on. can be
+        # an expression, must be quoted in list of quoting is required.
     *args, **kwargs):
         super().__init__(*args, **kwargs)
 
+        if columns_definition and exclude_columns:
+            raise Exception('Must not supply both columns_definition and ' \
+                + 'exclude_columns!')
+
         if not dwh_engine or not dwh_engine in EC.DWH_ENGINES:
             raise Exception('Invalid DWH Engine: {0}\n\nAccapted Engines:{1}'
                 .format(
@@ -88,6 +102,9 @@ class EWAHBaseOperator(BaseOperator):
                 '\n'.join(EC.DWH_ENGINES),
             ))
 
+        if index_columns and not dwh_engine == EC.DWH_ENGINE_POSTGRES:
+            raise Exception('Indices are only allowed for PostgreSQL DWHs!')
+
         if dwh_engine == EC.DWH_ENGINE_SNOWFLAKE:
             if not target_database_name:
                 conn_db_name = BaseHook.get_connection(dwh_conn_id)
@@ -110,6 +127,10 @@ class EWAHBaseOperator(BaseOperator):
                 raise Exception('This operator requires the argument ' \
                     + 'columns_definition!')
 
+        if primary_key_column_name and update_on_columns:
+            raise Exception('Cannot supply BOTH primary_key_column_name AND' + \
+                ' update_on_columns!')
+
         if not drop_and_replace:
             if not (
                 update_on_columns
@@ -147,6 +168,8 @@ class EWAHBaseOperator(BaseOperator):
         self.primary_key_column_name = primary_key_column_name # may be used ...
         #   ... by a child class at execution!
         self.add_metadata = add_metadata
+        self.exclude_columns = exclude_columns
+        self.index_columns = index_columns
 
         self.hook = get_dwhook(self.dwh_engine)
 
@@ -164,6 +187,27 @@ class EWAHBaseOperator(BaseOperator):
         self.upload_hook = self.hook(self.dwh_conn_id)
         # execute operator
         result = self.ewah_execute(context)
+        # if PostgreSQL and arg given: create indices
+        for column in self.index_columns:
+            # Use hashlib to create a unique 63 character string as index
+            # name to avoid breaching index name length limits and accidental
+            # duplicates / missing indices due to name truncation leading to
+            # identical index names.
+            self.hook.execute(self._INDEX_QUERY.format(
+                '__ewah_' + hashlib.blake2b(
+                    (self.target_schema_name
+                        + self.target_schema_suffix
+                        + '.'
+                        + self.target_table_name
+                        + '.'
+                        + column
+                    ).encode(),
+                    digest_size=28,
+                ).hexdigest(),
+                self.target_schema_name + self.target_schema_suffix,
+                self.target_table_name,
+                column,
+            ))
         # commit only at the end, so that no data may be committed before an
         # error occurs.
         self.log.info('Now committing changes!')
@@ -207,7 +251,9 @@ class EWAHBaseOperator(BaseOperator):
             if self.add_metadata and self._metadata:
                 datum.update(self._metadata)
             for field in datum.keys():
-                if not (result.get(field, {}).get(EC.QBC_FIELD_TYPE) \
+                if field in self.exclude_columns:
+                    datum[field] = None
+                elif not (result.get(field, {}).get(EC.QBC_FIELD_TYPE) \
                     == inconsistent_data_type) and (not datum[field] is None):
                     if result.get(field):
                         # column has been added in a previous iteration.
@@ -248,6 +294,20 @@ class EWAHBaseOperator(BaseOperator):
             # Note: This is also where metadata is added, if applicable
             columns_definition = self._create_columns_definition(data)
 
+        if self.update_on_columns:
+            pk_list = self.update_on_columns # is a list already
+        elif self.primary_key_column_name:
+            pk_list = [self.primary_key_column_name]
+        else:
+            pk_list = []
+
+        if pk_list:
+            for pk_name in pk_list:
+                if not pk_name in columns_definition.keys():
+                    raise Exception(('Column {0} does not exist but is ' + \
+                        'expected!').format(pk_name))
+                columns_definition[pk_name][EC.QBC_FIELD_PK] = True
+
         hook = self.upload_hook
 
         if (not self.drop_and_replace) or (self.upload_call_count > 1):
@@ -272,7 +332,8 @@ class EWAHBaseOperator(BaseOperator):
             data=data,
             columns_definition=columns_definition,
             table_name=self.target_table_name,
-            schema_name=self.target_schema_name+self.target_schema_suffix,
+            schema_name=self.target_schema_name,
+            schema_suffix=self.target_schema_suffix,
             database_name=self.target_database_name,
             drop_and_replace=self.drop_and_replace and \
                 (self.upload_call_count == 1), # In case of chunking of uploads
